@@ -55,6 +55,13 @@
 #include "clutter-stage-window.h"
 #include "clutter-version.h"
 
+#define CLUTTER_DISABLE_DEPRECATION_WARNINGS
+#include "deprecated/clutter-backend.h"
+
+#ifdef HAVE_CLUTTER_WAYLAND_COMPOSITOR
+#include "wayland/clutter-wayland-compositor.h"
+#endif /* HAVE_CLUTTER_WAYLAND_COMPOSITOR */
+
 #include <cogl/cogl.h>
 
 #ifdef CLUTTER_INPUT_X11
@@ -81,7 +88,9 @@
 #endif
 
 #ifdef HAVE_CLUTTER_WAYLAND_COMPOSITOR
+#include <cogl/cogl-wayland-server.h>
 #include <wayland-server.h>
+#include "wayland/clutter-wayland-compositor.h"
 #endif
 
 G_DEFINE_ABSTRACT_TYPE (ClutterBackend, clutter_backend, G_TYPE_OBJECT);
