@@ -284,6 +284,17 @@ clutter_vertex_equal (const ClutterVertex *vertex_a,
          fabsf (vertex_a->z - vertex_b->z) < FLOAT_EPSILON;
 }
 
+static void
+clutter_vertex_interpolate (const ClutterVertex *a,
+                            const ClutterVertex *b,
+                            double               progress,
+                            ClutterVertex       *res)
+{
+  res->x = a->x + (b->x - a->x) * progress;
+  res->y = a->y + (b->y - a->y) * progress;
+  res->z = a->z + (b->z - a->z) * progress;
+}
+
 static gboolean
 clutter_vertex_progress (const GValue *a,
                          const GValue *b,
@@ -294,9 +305,7 @@ clutter_vertex_progress (const GValue *a,
   const ClutterVertex *bv = g_value_get_boxed (b);
   ClutterVertex res;
 
-  res.x = av->x + (bv->x - av->x) * progress;
-  res.y = av->y + (bv->y - av->y) * progress;
-  res.z = av->z + (bv->z - av->z) * progress;
+  clutter_vertex_interpolate (av, bv, progress, &res);
 
   g_value_set_boxed (retval, &res);
 
@@ -1280,12 +1289,86 @@ clutter_rect_progress (const GValue *a,
 static gpointer
 clutter_matrix_copy (gpointer data)
 {
-  return g_memdup (data, sizeof (ClutterMatrix));
+  return cogl_matrix_copy (data);
 }
 
-G_DEFINE_BOXED_TYPE (ClutterMatrix, clutter_matrix,
-                     clutter_matrix_copy,
-                     clutter_matrix_free)
+static gboolean
+clutter_matrix_progress (const GValue *a,
+                         const GValue *b,
+                         gdouble       progress,
+                         GValue       *retval)
+{
+  const ClutterMatrix *matrix1 = g_value_get_boxed (a);
+  const ClutterMatrix *matrix2 = g_value_get_boxed (b);
+  ClutterVertex scale1 = CLUTTER_VERTEX_INIT (1.f, 1.f, 1.f);
+  float shear1[3] = { 0.f, 0.f, 0.f };
+  ClutterVertex rotate1 = CLUTTER_VERTEX_INIT_ZERO;
+  ClutterVertex translate1 = CLUTTER_VERTEX_INIT_ZERO;
+  ClutterVertex4 perspective1 = { 0.f, 0.f, 0.f, 0.f };
+  ClutterVertex scale2 = CLUTTER_VERTEX_INIT (1.f, 1.f, 1.f);
+  float shear2[3] = { 0.f, 0.f, 0.f };
+  ClutterVertex rotate2 = CLUTTER_VERTEX_INIT_ZERO;
+  ClutterVertex translate2 = CLUTTER_VERTEX_INIT_ZERO;
+  ClutterVertex4 perspective2 = { 0.f, 0.f, 0.f, 0.f };
+  ClutterVertex scale_res = CLUTTER_VERTEX_INIT (1.f, 1.f, 1.f);
+  float shear_res = 0.f;
+  ClutterVertex rotate_res = CLUTTER_VERTEX_INIT_ZERO;
+  ClutterVertex translate_res = CLUTTER_VERTEX_INIT_ZERO;
+  ClutterVertex4 perspective_res = { 0.f, 0.f, 0.f, 0.f };
+  ClutterMatrix res;
+
+  clutter_matrix_init_identity (&res);
+
+  _clutter_util_matrix_decompose (matrix1,
+                                  &scale1, shear1, &rotate1, &translate1,
+                                  &perspective1);
+  _clutter_util_matrix_decompose (matrix2,
+                                  &scale2, shear2, &rotate2, &translate2,
+                                  &perspective2);
+
+  /* perspective */
+  _clutter_util_vertex4_interpolate (&perspective1, &perspective2, progress, &perspective_res);
+  res.wx = perspective_res.x;
+  res.wy = perspective_res.y;
+  res.wz = perspective_res.z;
+  res.ww = perspective_res.w;
+
+  /* translation */
+  clutter_vertex_interpolate (&translate1, &translate2, progress, &translate_res);
+  cogl_matrix_translate (&res, translate_res.x, translate_res.y, translate_res.z);
+
+  /* rotation */
+  clutter_vertex_interpolate (&rotate1, &rotate2, progress, &rotate_res);
+  cogl_matrix_rotate (&res, rotate_res.x, 1.0f, 0.0f, 0.0f);
+  cogl_matrix_rotate (&res, rotate_res.y, 0.0f, 1.0f, 0.0f);
+  cogl_matrix_rotate (&res, rotate_res.z, 0.0f, 0.0f, 1.0f);
+
+  /* skew */
+  shear_res = shear1[2] + (shear2[2] - shear1[2]) * progress; /* YZ */
+  if (shear_res != 0.f)
+    _clutter_util_matrix_skew_yz (&res, shear_res);
+
+  shear_res = shear1[1] + (shear2[1] - shear1[1]) * progress; /* XZ */
+  if (shear_res != 0.f)
+    _clutter_util_matrix_skew_xz (&res, shear_res);
+
+  shear_res = shear1[0] + (shear2[0] - shear1[0]) * progress; /* XY */
+  if (shear_res != 0.f)
+    _clutter_util_matrix_skew_xy (&res, shear_res);
+
+  /* scale */
+  clutter_vertex_interpolate (&scale1, &scale2, progress, &scale_res);
+  cogl_matrix_scale (&res, scale_res.x, scale_res.y, scale_res.z);
+
+  g_value_set_boxed (retval, &res);
+
+  return TRUE;
+}
+
+G_DEFINE_BOXED_TYPE_WITH_CODE (ClutterMatrix, clutter_matrix,
+                               clutter_matrix_copy,
+                               clutter_matrix_free,
+                               CLUTTER_REGISTER_INTERVAL_PROGRESS (clutter_matrix_progress))
 
 /**
  * clutter_matrix_alloc:
@@ -1313,7 +1396,7 @@ clutter_matrix_alloc (void)
 void
 clutter_matrix_free (ClutterMatrix *matrix)
 {
-  g_free (matrix);
+  cogl_matrix_free (matrix);
 }
 
 /**
