@@ -239,7 +239,12 @@ clutter_backend_x11_create_device_manager (ClutterBackendX11 *backend_x11)
             {
 #ifdef HAVE_XINPUT_2
               int major = 2;
+
+#ifdef HAVE_XINPUT_2_2
+              int minor = 2;
+#else
               int minor = 0;
+#endif /* HAVE_XINPUT_2_2 */
 
               if (XIQueryVersion (backend_x11->xdpy, &major, &minor) != BadRequest)
                 {
@@ -250,6 +255,8 @@ clutter_backend_x11_create_device_manager (ClutterBackendX11 *backend_x11)
                                   "backend", backend_x11,
                                   "opcode", event_base,
                                   NULL);
+
+                  backend_x11->xi_minor = minor;
                 }
               else
 #endif /* HAVE_XINPUT_2 */
@@ -262,6 +269,7 @@ clutter_backend_x11_create_device_manager (ClutterBackendX11 *backend_x11)
                                   "event-base", first_event,
                                   NULL);
 
+                  backend_x11->xi_minor = -1;
                 }
             }
         }
@@ -274,6 +282,8 @@ clutter_backend_x11_create_device_manager (ClutterBackendX11 *backend_x11)
             g_object_new (CLUTTER_TYPE_DEVICE_MANAGER_X11,
                           "backend", backend_x11,
                           NULL);
+
+          backend_x11->xi_minor = -1;
         }
 
       backend = CLUTTER_BACKEND (backend_x11);
@@ -303,9 +313,9 @@ clutter_backend_x11_create_keymap (ClutterBackendX11 *backend_x11)
     }
 }
 
-gboolean
-_clutter_backend_x11_pre_parse (ClutterBackend  *backend,
-                                GError         **error)
+static gboolean
+clutter_backend_x11_pre_parse (ClutterBackend  *backend,
+                               GError         **error)
 {
   const gchar *env_string;
 
@@ -336,9 +346,9 @@ _clutter_backend_x11_pre_parse (ClutterBackend  *backend,
   return TRUE;
 }
 
-gboolean
-_clutter_backend_x11_post_parse (ClutterBackend  *backend,
-                                 GError         **error)
+static gboolean
+clutter_backend_x11_post_parse (ClutterBackend  *backend,
+                                GError         **error)
 {
   ClutterBackendX11 *backend_x11 = CLUTTER_BACKEND_X11 (backend);
   ClutterSettings *settings;
@@ -804,8 +814,8 @@ clutter_backend_x11_class_init (ClutterBackendX11Class *klass)
 
   backend_class->stage_window_type = CLUTTER_TYPE_STAGE_X11;
 
-  backend_class->pre_parse = _clutter_backend_x11_pre_parse;
-  backend_class->post_parse = _clutter_backend_x11_post_parse;
+  backend_class->pre_parse = clutter_backend_x11_pre_parse;
+  backend_class->post_parse = clutter_backend_x11_post_parse;
   backend_class->add_options = clutter_backend_x11_add_options;
   backend_class->get_features = clutter_backend_x11_get_features;
 
@@ -1175,6 +1185,8 @@ clutter_x11_get_input_devices (void)
   ClutterDeviceManager *manager;
 
   manager = clutter_device_manager_get_default ();
+  if (manager == NULL)
+    return NULL;
 
   return clutter_device_manager_peek_devices (manager);
 }
